@@ -42,13 +42,14 @@ typedef enum {
     BDJ_EVENT_AUDIO_STREAM,
     BDJ_EVENT_SECONDARY_STREAM,
     BDJ_EVENT_UO_MASKED,
+    BDJ_EVENT_SEEK,
 } BDJ_EVENT;
 
 typedef struct {
-    char *persistent_root;
-    char *cache_root;
+    char *persistent_root;   /* BD-J Xlet persistent storage */
+    char *cache_root;        /* BD-J binding unit data area */
 
-    char *classpath;
+    char *classpath;         /* BD-J implementation class path (location of libbluray.jar) */
 } BDJ_STORAGE;
 
 typedef struct bdjava_s BDJAVA;
