@@ -23,14 +23,12 @@ import javax.tv.service.*;
 
 public abstract class ServiceDetailsSIChangeEvent extends SIChangeEvent {
     public ServiceDetailsSIChangeEvent(ServiceDetails details,
-            SIChangeType type, SIElement element)
-    {
+            SIChangeType type, SIElement element) {
         super(details, type, element);
         this.details = details;
     }
 
-    public ServiceDetails getServiceDetails()
-    {
+    public ServiceDetails getServiceDetails() {
         return details;
     }
 
