@@ -24,14 +24,12 @@ import javax.tv.service.navigation.ServiceDetails;
 
 public class ServiceDetailsChangeEvent extends TransportSIChangeEvent {
     public ServiceDetailsChangeEvent(Transport transport, SIChangeType type,
-            ServiceDetails details)
-    {
+            ServiceDetails details) {
         super(transport, type, details);
         this.details = details;
     }
 
-    public ServiceDetails getServiceDetails()
-    {
+    public ServiceDetails getServiceDetails() {
         return details;
     }
 
