@@ -22,8 +22,8 @@ package org.blurayx.s3d.media;
 import java.io.Serializable;
 import java.util.EventObject;
 
-public class AlignedSubtitleChangeEvent extends EventObject implements Serializable
-{
+public class AlignedSubtitleChangeEvent extends EventObject implements Serializable {
+
     private int subtitleAlignment;
 
     public AlignedSubtitleChangeEvent(AlignedSubtitleControl source, int subtitleAlignment) {
@@ -34,4 +34,4 @@ public class AlignedSubtitleChangeEvent extends EventObject implements Serializa
     public int getAlignedSubtitle() {
         return subtitleAlignment;
     }
-}
\ No newline at end of file
+}
