@@ -19,25 +19,29 @@
 package org.davic.net;
 
 public class Locator implements javax.tv.locator.Locator {
-    public Locator(String url)
-    {
+    public Locator(String url) {
         this.url = url;
     }
 
-    public String toString()
-    {
+    public String toString() {
         return url;
     }
 
-    public boolean hasMultipleTransformations()
-    {
+    public boolean hasMultipleTransformations() {
         return false;
     }
 
-    public String toExternalForm()
-    {
+    public String toExternalForm() {
         return url;
     }
 
-    private String url;
+    public boolean equals(Object obj) {
+        if (obj == null || !(obj instanceof Locator) || url == null)
+            return false;
+
+        Locator locator = (Locator)obj;
+        return toExternalForm().equals(locator.toExternalForm());
+    }
+
+    protected String url;
 }
