@@ -23,10 +23,12 @@
 #ifndef BD_BITS_H
 #define BD_BITS_H
 
-#include "file/file.h"
+#include "util/attributes.h"
+#include "file/filesystem.h" // BD_FILE_H
+
+#include <stdint.h>
+#include <stddef.h>    // size_t
 
-#include <unistd.h>
-#include <stdio.h>
 
 /**
  * \file
@@ -36,11 +38,11 @@
 #define BF_BUF_SIZE   (1024*32)
 
 typedef struct {
-    uint8_t *p_start;
-    uint8_t *p;
-    uint8_t *p_end;
+    const uint8_t *p_start;
+    const uint8_t *p;
+    const uint8_t *p_end;
 
-    ssize_t  i_left;    /* i_count number of available bits */
+    int            i_left;    /* i_count number of available bits */
 } BITBUFFER;
 
 typedef struct {
@@ -52,12 +54,14 @@ typedef struct {
     size_t     size;
 } BITSTREAM;
 
-BD_PRIVATE void bb_init( BITBUFFER *bb, uint8_t *p_data, size_t i_data );
+BD_PRIVATE void bb_init( BITBUFFER *bb, const uint8_t *p_data, size_t i_data );
 BD_PRIVATE void bs_init( BITSTREAM *bs, BD_FILE_H *fp );
 BD_PRIVATE void bb_seek( BITBUFFER *bb, int64_t off, int whence);
 BD_PRIVATE void bs_seek( BITSTREAM *bs, int64_t off, int whence);
-BD_PRIVATE void bb_skip( BITBUFFER *bb, ssize_t i_count );
-BD_PRIVATE void bs_skip( BITSTREAM *bs, ssize_t i_count );
+//BD_PRIVATE void bb_seek_byte( BITBUFFER *bb, int64_t off);
+BD_PRIVATE void bs_seek_byte( BITSTREAM *s, int64_t off);
+BD_PRIVATE void bb_skip( BITBUFFER *bb, size_t i_count );
+BD_PRIVATE void bs_skip( BITSTREAM *bs, size_t i_count );  /* note: i_count must be less than BF_BUF_SIZE */
 BD_PRIVATE uint32_t bb_read( BITBUFFER *bb, int i_count );
 BD_PRIVATE uint32_t bs_read( BITSTREAM *bs, int i_count );
 
@@ -92,16 +96,6 @@ static inline int64_t bs_avail( const BITSTREAM *bs )
     return bs_end(bs) - bs_pos(bs);
 }
 
-static inline void bb_seek_byte( BITBUFFER *bb, int64_t off)
-{
-    bb_seek(bb, off << 3, SEEK_SET);
-}
-
-static inline void bs_seek_byte( BITSTREAM *s, int64_t off)
-{
-    bs_seek(s, off << 3, SEEK_SET);
-}
-
 static inline void bb_read_bytes( BITBUFFER *bb, uint8_t *buf, int i_count )
 {
     int ii;
